/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  chart2.pl -- une autre implementation d'un algorithme a charte
 *
 * ----------------------------------------------------------------
 * Description
 * une implementation d'un algorithme a charte de type Earley
 * en s'appuyant sur un modle ask/tell
 *
 * ----------------------------------------------------------------
 */

%% Bibliothque pour les impressions
:-require 'format.pl'.

%% Bibliotheque pour les mta-call
%% :-require 'call.pl'.

:-extensional 'C'/3.

'C'([T|R],T,R).

:-extensional (-->)/2.
:-extensional verbose_opt.

:-rec_prolog verbose/2.

verbose(Msg,Args) :-
   argv(L),
   ( verbose_opt ->
       format(Msg,Args)
   ;   
       true
   ).

earley_parse(Goal,L,R) :-
	( prediction(Goal(L)),
	  fail
	;
	  process,
	  fail
	;
	  stored((Goal --> true)(L,R))
	).

prediction(A(I)) :-
	recorded((A --> Body)),
	dcg_normalize(Body,XBody),
	store((A --> XBody)(I,I)).

process :-
	stored((A-->B,Beta)(I,J)),
	( prediction(B(J))
	;
	  stored((B-->true)(J,K)),
	  store((A --> Beta)(I,K))
	)
	.

process :-
	stored((A-->B::[_|_],Beta)(I,J)),
	scan(B,J,K),	
	store((A --> Beta)(I,K)).

/*
process :-
	stored((A-->B::{G},Beta)(I,J)),
	format('Trying escape on ~w\n',[G]),
	%%     call!pcall(G),
	G,
	format('=> Got ~w\n',[G]),
	store((A --> Beta)(I,J)).
*/

:-rec_prolog scan/3.

scan([],L,L).
scan([Token|Tokens],L,R) :-
	'C'(L,Token,M),
	scan(Tokens,M,R).

%% stored(Item)
%% attend (mode ask) la prsence de Item dans la table
stored(Item) :-
	'$answers'(store(Item)),
	verbose('got stored ~w\n',[Item]).

%% store(Item)
%% enregistre Item dans la table en mode tell
store(Item) :-
	verbose('storing ~w\n',[Item]).

:-rec_prolog dcg_normalize/2.

dcg_normalize(Body,XBody) :-
	( Body = (B,Rest) ->
	    dcg_normalize(Rest,XRest),
	    XBody = (B,XRest)
	;
	    XBody = (Body,true)
	).

:-rec_prolog get_sentence/1.
get_sentence(Sentence) :-
	argv(L),
	( L=['-verbose'|Sentence] -> record(verbose_opt) ;  L = Sentence).

%% La requte,
%% ./chart2 french.dcg -- Paul mange une pomme
%% ./chart2 french.dcg -- -verbose Paul mange une pomme
?- get_sentence(Sentence),
    earley_parse(s,Sentence,[]).

:-rec_prolog lexicon/2.

:-public(lexicon/2).

%% lexicon(Form,Struct)
